/***********************************************************************************
*	bit_win.h -	Bit manipulation macros.
*
* Author	:	David Rokhvarg	<davidr@sangoma.com>
*
* Copyright	:	(c) 2000 Sangoma Technologies Inc.
*
************************************************************************************/

#ifndef _BIT_WIN_H_
#define _BIT_WIN_H_

#if !defined(WAN_KERNEL)
/* Windows user-space */

/* internal macros */
/* byte of the bitstring bit is in */
#define	_bit_byte(bit)	((bit) >> 3)

/* mask for the bit within its byte */
#define	_bit_mask(bit)	(1 << ((bit)&0x7))

/* external macros */
/* bytes in a bitstring of nbits bits */
#define	bitstr_size(nbits)	((((nbits) - 1) >> 3) + 1)

/* is bit N of bitstring name set? */
#define	bit_test(name, bit) \
	((name)[_bit_byte(bit)] & _bit_mask(bit))

/* set bit N of bitstring name */
#define	bit_set(name, bit) \
	(name)[_bit_byte(bit)] |= _bit_mask(bit)

/* clear bit N of bitstring name */
#define	bit_clear(name, bit) \
	(name)[_bit_byte(bit)] &= ~_bit_mask(bit)

static __inline int test_and_set_bit(int bit, void* name)
{
	if(bit_test((unsigned char*)name, bit)){
		/* bit already set. say so. */
		return 1;
	}else{
		/* not set. set it. */
		bit_set((unsigned char*)name, bit);
		return 0;
	}
}

static __inline int test_and_clear_bit(int bit, void* name)
{
	if(bit_test((unsigned char*)name, bit)){
		/* bit is set. clear it */
		bit_clear((unsigned char*)name, bit);
		return 1;
	}else{
		/* not set. say so. */
		return 0;
	}
}

#else
/* Windows kernel-space */

# define bit_test(Dest, BitNum)	\
	(InterlockedExchangeAdd((PLONG)(Dest), 0) & (1 << BitNum))

# define bit_set(Dest, BitNum)	\
	InterlockedOr((PLONG)(Dest), (1 << BitNum))

# define bit_clear(Dest, BitNum)	\
	InterlockedAnd((PLONG)(Dest), (~(1 << BitNum)))

# define test_and_set_bit(BitNum, Dest)	\
	( InterlockedOr((PLONG)(Dest), (1 << BitNum)) & (1 << BitNum) )

# define test_and_clear_bit(BitNum, Dest)	\
	( InterlockedAnd((PLONG)(Dest), (~(1 << BitNum))) & (1 << BitNum) )

#endif

#define	test_bit(bit, name)	bit_test((unsigned char*)name, bit)

#define set_bit(bit, name)	bit_set((unsigned char*)name, bit)

#define clear_bit(bit, name)	bit_clear((unsigned char*)name, bit)


/************* wan bit macros ************/

#define wan_clear_bit(a,b)  		clear_bit((a),(unsigned long*)(b))
#define wan_set_bit(a,b)    		set_bit((a),(unsigned long*)(b))
#define wan_test_bit(a,b)   		test_bit((a),(unsigned long*)(b))
#define wan_test_and_set_bit(a,b)	test_and_set_bit((a),(unsigned long*)(b))
#define wan_test_and_clear_bit(a,b)	test_and_clear_bit((a),(unsigned long*)(b))

/*****************************************/

/*!
 \fn get_bit_number()
 \brief Retrieve the 0-based bit number of the first bit which is set.
		If more than one bit is set in 'bit_map', all the other bits
		ignored.
 \param bit_map	The bit map which should contain at most one bit set
 \return -1: no bit is set, else bit number
*/
static
int
wp_get_bit_number(
	unsigned int bit_map
	)
{
	int i, bit_number = -1;

	for(i = 0; i < sizeof(bit_map) * 8; i++){
		if(bit_map & ( 1 << i )){
			bit_number = i;
			break;
		}
	}

	return bit_number;
}

#endif /* _BIT_WIN_H_ */

